from typing import List
from dataclasses import dataclass
from mlplatform_lib.dataclass import ColumnOption, RowOption


@dataclass
class SelectorProperty:
    column_select_options: List[ColumnOption]
    row_select_options: List[RowOption]



import os
from mlplatform_lib.api_client import RunMode


class Auth:
    SECRET_LIST = ["projectId", "userId", "accessToken", "refreshToken", "authorizationType"]

    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, "instance"):
            cls.instance = super(Auth, cls).__new__(cls)
        return cls.instance

    def __init__(self, *args, **kwargs):
        api_client = kwargs.get("api_client")

        if api_client.run_mode == RunMode.LOCAL:
            self.secret_mount_path = None
            self.authorization = api_client.Authorization
            self.projectId = api_client.projectId
            self.userId = api_client.userId
            self.refreshToken = api_client.refreshToken
        else:
            self.secret_mount_path = os.environ["secretMountPath"]
            self.load_env(self.secret_mount_path)

    def load_env(self, secret_mount_path):
        file_list = [f for f in os.listdir(self.secret_mount_path)]
        for secret in self.SECRET_LIST:
            if secret not in file_list:
                print(f'Cannot found "{secret}" in secret path "{self.secret_mount_path}".')
                exit(1)

        for file in file_list:
            if file in self.SECRET_LIST:
                setattr(self, file, open(os.path.join(self.secret_mount_path, file), "r").read())
        self.authorization = self.accessToken



from dataclasses import dataclass
from enum import Enum
import json
import requests
from typing import Optional, Dict, List, Union, Tuple
from pathlib import Path

class ProauthRequestException(Exception):
    status_code = 200

    def __init__(self, message, status_code=None):
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code

class ProauthRequestType(Enum):
    CREATE = 0
    READ = 1
    UPDATE = 2
    DELETE = 3

@dataclass
class ProauthUserAuth:
    refresh_token: str
    access_token: str
    result: str

@dataclass
class ProauthRequestResult:
    data: Optional[Union[Dict, List[Dict], bytes]]
    content: Optional[str]
    status_code: Optional[int]

class ProauthHttpClient:
    def __init__(self, proauth_addr):
        self.base_url = proauth_addr + "/proauth/oauth"

    def send_request(
        self,
        service: str,
        rest: Optional[dict],
        query: Optional[dict],
        data: Union[Optional[dict], Path],
        request_type: ProauthRequestType,
    ) -> ProauthRequestResult:
        headers = {
            "Content-Type": "application/json"
        }

        if rest is None:
            rest = {}

        if query is None:
            query = {}

        if data is None:
            data = {}

        url = self.base_url
        for key, val in rest.items():
            url = url + "/" + str(key) + "/" + str(val)
        if service != "" and service is not None:
            url = url + "/" + service

        if len(query) != 0:
            url += "?"
            for idx, (key, val) in enumerate(query.items()):
                if idx != 0:
                    url += "&"
                url = url + str(key) + "=" + str(val)

        if request_type == ProauthRequestType.READ:
            response = requests.get(url, headers=headers, json=data, verify=False)
        elif request_type == ProauthRequestType.CREATE:
            if isinstance(data, Path):
                headers.pop("Content-Type", None)
                response = requests.post(url, headers=headers, files={"file": open(data, "rb")})
            else:
                response = requests.post(url, headers=headers, json=data, verify=False)
        elif request_type == ProauthRequestType.UPDATE:
            response = requests.put(url, headers=headers, json=data, verify=False)
        elif request_type == ProauthRequestType.DELETE:
            response = requests.delete(url, headers=headers, json=data, verify=False)
        else:
            raise ProauthRequestException(status_code=405, message=f"{request_type} method not allowed.")

        if response.status_code == 200:
            return ProauthRequestException(response.json(), response.status_code)
        else:
            print(
                (
                    f"{service} {request_type.value} failed\n"
                    f"status_code: {response.status_code}.\n"
                    f"reason: {response.text}\n"
                )
            )
            raise ProauthRequestException(response.content.decode(), response.status_code)


    def update_token(self, refresh_token: str) -> Tuple[str, str, str]:
        print("===proauth_http_client update_token()=====")
        result = self.send_request(
            "authenticate",
            {},
            {},
            {"refresh_token" : refresh_token},
            ProauthRequestType.UPDATE
        )
        return result.result, result.refresh_token, result.access_token


from .inference_dto import InferenceDto
from .inference_predefinedai_dto import InferencePredefinedaiDto
from .inference_automl_dto import InferenceAutomlDto


__all__ = ["InferenceDto", "InferencePredefinedaiDto", "InferenceAutomlDto"]



from dataclasses import dataclass, field
from mlplatform_lib.dataclass.experiment.type import ExperimentType


@dataclass
class InferenceDto:
    id: int = field(init=False, default=0)
    experiment_type: ExperimentType = field(init=False)
    inference_path: str



from dataclasses import dataclass


@dataclass
class ExperimentAutomlDto:
    id: int
    authentication_id: int
    experiment_type: str
    name: str
    namespace: str
    pvc_name: str
    pvc_size: str
    created_on: str
    description: str
    do_id: int
    target_column: str
    task_type: str
    metric: str
    time_column: str
    group_columns: str



from typing import Optional
from dataclasses import dataclass
from mlplatform_lib.dataclass import SelectorProperty


@dataclass
class FileDownload:
    file_name: str
    file_type: str
    selector_property: SelectorProperty
    file_token: Optional[str]



import os
from typing import List, Optional

import pandas as pd

from mlplatform_lib.api_client import ApiClient, RunMode
from mlplatform_lib.automl.automl_http_client import AutomlHttpClient
from mlplatform_lib.dataclass import InsertTupleObject
from mlplatform_lib.dataclass.experiment.type import ExperimentType
from mlplatform_lib.dataclass.model import ModelInfoDto, ModelAutomlDto, ModelAutomlBestDto
from mlplatform_lib.dataclass.model.type import ModelStatus
from mlplatform_lib.hyperdata import HyperdataApi


class AutomlApi:
    def __init__(self, api_client: ApiClient = None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        if api_client.run_mode == RunMode.KUBERNETES:
            self.automl_client = AutomlHttpClient(
                mlplatform_addr=os.environ["mlplatformAddr"], api_client=self.api_client
            )
            self.hyperdata_api = HyperdataApi(api_client=self.api_client)
        self.experiment_id = self.api_client.experiment_id
        self.train_id = self.api_client.train_id
        self.inference_id = self.api_client.inference_id
        # self.target_do_id = os.environ["targetDoId"] if "targetDoId" in os.environ else None

    def get_models(self) -> Optional[List[ModelAutomlDto]]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.automl_client.get_models(experiment_id=self.experiment_id, train_id=self.train_id,)
        else:
            print("Current mode is local, Skip get_models.")
            return None

    def insert_model(self, model: ModelAutomlDto) -> Optional[ModelAutomlDto]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            model.train_id = self.train_id
            model.status = ModelStatus.SUCCESS
            model.experiment_type = ExperimentType.AUTOML
            return self.automl_client.insert_model(
                experiment_id=self.experiment_id, train_id=self.train_id, model=model,
            )
        else:
            print("Current mode is local, Skip insert_model.")
            model.id = 1
            model.train_id = self.train_id
            model.status = ModelStatus.SUCCESS
            model.experiment_type = ExperimentType.AUTOML
            return model

    def insert_model_info(self, model_id: int, model_info: ModelInfoDto) -> ModelInfoDto:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            model_info.status = ModelStatus.SUCCESS
            return self.automl_client.insert_model_info(
                experiment_id=self.experiment_id,
                train_id=self.train_id,
                model_id=model_id,
                model_info=model_info,
            )
        else:
            print("Current mode is local, Skip insert_visualizations.")

    def _get_inference_result_table_name(self) -> str:
        automl_experiment_dto = self.automl_client.get_experiment(experiment_id=self.experiment_id)

        return f"AUTOML_{automl_experiment_dto.name}_{str(automl_experiment_dto.id)}"

    def upload_inference_csv(self, inference_csv_path: str):
        if self.api_client.run_mode == RunMode.KUBERNETES:
            inference_data = pd.read_csv(inference_csv_path)

            table_name = self._get_inference_result_table_name()
            self.target_do_id = self.hyperdata_api.create_inference_result(
                table_name=table_name, object_name=table_name, columns=inference_data.columns.tolist()
            )

            insert_tuple_object = InsertTupleObject(
                isTruncated="True",
                targetColNames=inference_data.columns.tolist(),
                tableData=inference_data.values.tolist(),
            )

            self.hyperdata_api.hyperdata_client.insert_dataobject_tuple(
                dataobject_id=self.target_do_id, insert_tuple_objects=insert_tuple_object,
            )

            automl_inference_dto = self.automl_client.get_inference(
                experiment_id=self.experiment_id, inference_id=self.inference_id
            )

            automl_inference_dto.target_do_id = self.target_do_id
            self.automl_client.update_inference(
                experiment_id=self.experiment_id, inference_automl_dto=automl_inference_dto,
            )
            os.remove(inference_csv_path)
        else:
            print("Current mode is local, Skip upload_inference_csv.")

    def insert_model_automl_best(self, model_automl_best: ModelAutomlBestDto) -> ModelAutomlBestDto:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.automl_client.insert_model_automl_best(
                experiment_id=self.experiment_id, train_id=self.train_id, model_automl_best=model_automl_best,
            )
        else:
            print("Current mode is local, Skip insert_model_automl_best.")



from mlplatform_lib.dataclass import (
    DataObject,
    TableDescInfo,
    InsertTupleObject,
    TableFromDefaultDataSourceInfo,
    DataObjectInfoList,
    DbSourceDataObjectCandidate,
    DbSourceDataObjectCandidateList,
    FileDownload,
    SelectorProperty,
    ColumnOption,
)
from dataclasses import dataclass
from enum import Enum
import json
import requests
from typing import Optional, Dict, List, Union, Tuple
from mlplatform_lib.utils.dataclass_utils import from_dict, to_dict
from mlplatform_lib.api_client import ApiClient
from mlplatform_lib.auth import Auth
from mlplatform_lib.proauth.proauth_http_client import ProauthHttpClient
import base64


HYPERDATA_DATETIME_FORMAT = "%Y.%m.%d %H.%M.%S"
HYPERDATA_DATETIME_INIT_STR = "1970.01.01 00.00.00"


class HyperdataRequestException(Exception):
    status_code = 200

    def __init__(self, message, status_code=None):
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code


class HyperdataRequestType(Enum):
    CREATE = 0
    READ = 1
    UPDATE = 2
    DELETE = 3
    DOWNLOAD = 4
    READ_WITH_FORM_DATA = 5


@dataclass
class HyperdataUserAuth:
    project_id: str
    user_id: str
    authorization: str


@dataclass
class HyperdataRequestResult:
    data: Optional[Union[Dict, List[Dict], bytes]]
    content: Optional[str]
    status_code: Optional[int]


class HyperdataHttpClient:
    def __init__(self, hd_addr, api_client: ApiClient):
        self.base_url = hd_addr + "/hyperdata/web-service"
        self.api_client = api_client
        self.proauth_client = ProauthHttpClient(proauth_addr=api_client.proauth_addr)

    def send_request(
        self,
        service: str,
        rest: Optional[dict],
        query: Optional[dict],
        data: Optional[dict],
        request_type: HyperdataRequestType,
    ) -> HyperdataRequestResult:
        auth = Auth(api_client=self.api_client)
        headers = {
            "Content-Type": "application/json",
            "Authorization": auth.authorization,
            "userId": auth.userId,
        }

        if rest is None:
            rest = {}

        if query is None:
            query = {}

        if data is None:
            data = {}

        if service == "dataObjectTuple":
            headers["doType"] = "inferenceResult"

        if request_type == HyperdataRequestType.DOWNLOAD:
            headers["Content-Type"] = "application/octet-stream"

        url = self.base_url + "/projects/" + str(auth.projectId)
        for key, val in rest.items():
            url = url + "/" + str(key) + "/" + str(val)
        if service != "" and service is not None:
            url = url + "/" + service

        if len(query) != 0:
            url += "?"
            for idx, (key, val) in enumerate(query.items()):
                if idx != 0:
                    url += "&"
                url = url + str(key) + "=" + str(val)

        if data != "" and data is not None:
            data = {"dto": data}
        else:
            data = {"dto": ""}

        if request_type == HyperdataRequestType.READ or request_type == HyperdataRequestType.DOWNLOAD:
            response = requests.get(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == HyperdataRequestType.CREATE:
            response = requests.post(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == HyperdataRequestType.UPDATE:
            response = requests.put(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == HyperdataRequestType.DELETE:
            response = requests.delete(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == HyperdataRequestType.READ_WITH_FORM_DATA:
            response = requests.get(url, headers=headers, data=json.dumps(data), verify=False)
        else:
            return HyperdataRequestResult(None, None, None)

        if response.status_code == 200:
            if request_type == HyperdataRequestType.DOWNLOAD:
                return HyperdataRequestResult(response.content, None, response.status_code)
            else:
                return HyperdataRequestResult(
                    response.json(), response.content.decode(), response.status_code
                )
        else:
            if response.json()["exception"]["code"] == "HDE-0403":
                result, refresh_token, access_token = self.proauth_client.update_token(auth.refreshToken)
                if result == "true":
                    auth.refreshToken = refresh_token
                    auth.accessToken = access_token
                    auth.authorization = access_token
                    headers["Authorization"] = auth.accessToken

                    if (
                        request_type == HyperdataRequestType.READ
                        or request_type == HyperdataRequestType.DOWNLOAD
                    ):
                        response = requests.get(url, headers=headers, data=json.dumps(data), verify=False)
                    elif request_type == HyperdataRequestType.CREATE:
                        response = requests.post(url, headers=headers, data=json.dumps(data), verify=False)
                    elif request_type == HyperdataRequestType.UPDATE:
                        response = requests.put(url, headers=headers, data=json.dumps(data), verify=False)
                    elif request_type == HyperdataRequestType.DELETE:
                        response = requests.delete(url, headers=headers, data=json.dumps(data), verify=False)
                    elif request_type == HyperdataRequestType.READ_WITH_FORM_DATA:
                        response = requests.get(url, headers=headers, data=json.dumps(data), verify=False)
                    else:
                        return HyperdataRequestResult(None, None, None)

                else:
                    raise HyperdataRequestException(response.content.decode(), response.status_code)

            else:
                print(response.content.decode())
                print(
                    (
                        f"cannot connect to {self.base_url}. "
                        f"hyperdata server returns status code {response.status_code}"
                    )
                )
                raise HyperdataRequestException(response.content.decode(), response.status_code)

    # TODO: hyperdata datasource에 do 정보들을 다가지고 오는 so 구현 필요
    def get_do_list(self) -> List[DataObject]:
        result = self.send_request("automlExperimentDataObjectList", {}, {}, {}, HyperdataRequestType.READ)
        result_data = result.data["dto"]["dataObjectInfoList"]

        do_list = []
        for do_info in result_data:
            do_list.append(self.get_do_info(do_info["id"]))
        return do_list

    def get_do_simple_list(self) -> dict:
        result = self.send_request("automlExperimentDataObjectList", {}, {}, {}, HyperdataRequestType.READ)
        result_data = result.data["dto"]["dataObjectInfoList"]
        do_list = {}
        for do_info in result_data:
            do_list[do_info["name"]] = do_info["id"]
        return do_list

    def get_do_list_by_datasource_id(self, datasource_id: int) -> DataObjectInfoList:
        result = self.send_request(
            "dataobjects", {"datasources": datasource_id}, {}, {}, HyperdataRequestType.READ
        )
        result_data = result.data["dto"]["dataObjectInfoList"]
        return DataObjectInfoList(
            data_object_info_list=[from_dict(DataObject, do_info) for do_info in result_data]
        )

    def get_do_info(self, do_id: int) -> DataObject:
        result = self.send_request(
            "", {"dataobjects": do_id}, {"action": "Detail"}, {}, HyperdataRequestType.READ,
        )
        return from_dict(DataObject, result.data["dto"])

    #    def get_do_samples(self, dataobject_id: int, sample_size: int):
    #        return self.send_request(
    #            "",
    #            {"dataobjects": dataobject_id},
    #            {"action": "Sample", "size": sample_size},
    #            {},
    #            auth,
    #            HyperdataRequestType.READ,
    #        )

    # 아직 공식 hyperdata so에서 값이 이상하게 들어와 직접 만든걸 사용
    def get_do_samples(self, dataobject_id: int, sample_size: int) -> HyperdataRequestResult:
        result = self.send_request(
            "automlExperimentDataObjectSample",
            {"dataobjects": dataobject_id},
            {"size": sample_size},
            {},
            HyperdataRequestType.READ,
        )
        fixed_table_string = []
        for row in json.loads(result.data["dto"]["tableString"]):
            fixed_row = {}
            for k, v in row.items():
                fixed_row[k[1:-1]] = v
            fixed_table_string.append(fixed_row)
        result.data["dto"]["tableString"] = json.dumps(fixed_table_string)
        return result

    def get_do_detail_info(self, do_id: int, subtype: str = "Table"):
        result = self.send_request(
            "",
            {"isedit": 0, "subtype": subtype, "dataobjects": do_id},
            {"action": "Desc"},
            {},
            HyperdataRequestType.READ,
        )
        return from_dict(TableDescInfo, result.data["dto"])

    def download_do_to_csv(self, do_id: int) -> Tuple[HyperdataRequestResult, str, str]:
        # 1. get do info
        data_object = self.get_do_info(do_id=do_id)
        # 2. get do detail info
        table_desc_info = self.get_do_detail_info(do_id=do_id, subtype=data_object.subtype)

        # 3. create do to csv in hyperdata
        file_download = FileDownload(
            file_name=data_object.name,
            file_type="csv",
            selector_property=SelectorProperty(
                column_select_options=[
                    ColumnOption(name=col_info.name, alias=col_info.name, type="VARCHAR")
                    for col_info in table_desc_info.col_info_list
                ],
                row_select_options=[],
            ),
            file_token=None,
        )

        result = self.send_request(
            "",
            {"dataobjects": do_id},
            {"action": "FileCreateWithOptions"},
            to_dict(file_download),
            HyperdataRequestType.CREATE,
        )
        file_download = from_dict(FileDownload, result.data["dto"])

        auth = Auth(api_client=self.api_client)
        download_info = (
            f"Authorization::{auth.authorization}"
            f"::userId::{auth.userId}"
            f"::dataObjectId::{data_object.id}"
            f"::filePath::{data_object.name}.csv"
            f"::fileToken::{file_download.file_token}"
        )

        # 4. download csv file
        download_info_encoded_str = base64.b64encode(download_info.encode("utf-8")).decode("utf-8")
        return (
            self.send_request(
                "",
                {"dataobjects": download_info_encoded_str},
                {"action": "FileSecureDownload"},
                {},
                HyperdataRequestType.DOWNLOAD,
            ),
            ",",
            "\\n",
        )

    def get_default_datasource_id(self) -> int:
        result = self.send_request("dataSourceDefault", {}, {}, {}, HyperdataRequestType.READ)
        return int(result.data["dto"]["defaultDataSourceId"])

    def get_dataobject_tables(self, datasource_id: int):
        result = self.send_request(
            "", {"datasources": datasource_id}, {"action": "Desc"}, {}, HyperdataRequestType.READ
        )

        result_data = result.data["dto"]["DbSourceDataObjectCandidate"]
        return DbSourceDataObjectCandidateList(
            db_source_data_object_candidate=[
                from_dict(DbSourceDataObjectCandidate, table_info) for table_info in result_data
            ]
        )

    def create_dataobject_table(
        self, datasource_id: int, table_from_default_data_source: TableFromDefaultDataSourceInfo,
    ) -> None:
        self.send_request(
            "dataobjects",
            {"datasources": datasource_id},
            {"action": "CreateTable"},
            to_dict(table_from_default_data_source),
            HyperdataRequestType.CREATE,
        )

    def create_dataobjects(self, datasource_id: int, dataobjects: DataObjectInfoList) -> DataObjectInfoList:
        result = self.send_request(
            "dataobjects",
            {"datasources": datasource_id},
            {},
            to_dict(dataobjects),
            HyperdataRequestType.CREATE,
        )
        result_data = result.data["dto"]["dataObjectInfoList"]
        return DataObjectInfoList(
            data_object_info_list=[from_dict(DataObject, do_info) for do_info in result_data]
        )

    def insert_dataobject_tuple(
        self, dataobject_id: int, insert_tuple_objects: InsertTupleObject,
    ) -> HyperdataRequestResult:
        return self.send_request(
            "dataObjectTuple",
            {"dataobjects": dataobject_id},
            {},
            {"jsonString": json.dumps(insert_tuple_objects.__dict__)},
            HyperdataRequestType.UPDATE,
        )

    def delete_dataobjects(self, dataobject_id: int) -> HyperdataRequestResult:
        return self.send_request(
            "datasources",
            {},
            {},
            {"dataobjectId": [dataobject_id], "datasourceId": [], "fileType": []},
            HyperdataRequestType.DELETE,
        )



from typing import Dict, List
import inspect
import inflection
from dataclasses import is_dataclass


# dataclass field(init=False)로 지정되어 있는 변수들은
# constructor에 넣어서 생성 할 수 없다.
# 따라서, 생성자에 넘겨줄 dictionary와 setattr로 설정할 변수를 나누어 설정한다.
def from_dict(dataclass, dict: Dict):
    sig = inspect.signature(dataclass.__init__)

    init_dict = {}
    init_false_dict = {}
    for key, val in dict.items():
        # camel case to snake case
        key = inflection.underscore(key)

        if key in sig.parameters.keys():
            key_class = sig.parameters[key].annotation
            if is_dataclass(key_class):
                init_dict[key] = from_dict(sig.parameters[key].annotation, val)
            # regard as typing hint
            elif hasattr(key_class, "__origin__") and (
                key_class.__origin__ == list or key_class.__origin__ == List
            ):
                if is_dataclass(sig.parameters[key].annotation.__args__[0]):
                    init_dict[key] = [
                        from_dict(sig.parameters[key].annotation.__args__[0], elem) for elem in val
                    ]
                else:
                    init_dict[key] = val

            else:
                init_dict[key] = val
        else:
            init_false_dict[key] = val

    instance = dataclass(**init_dict)
    for key, val in init_false_dict.items():
        setattr(instance, key, val)
    return instance


def to_dict(instance):
    result = {}
    for k, v in instance.__dict__.items():
        if is_dataclass(v):
            result[inflection.camelize(k, False)] = to_dict(v)
        elif isinstance(v, list):
            result[inflection.camelize(k, False)] = []
            for v_elem in v:
                if is_dataclass(v_elem):
                    result[inflection.camelize(k, False)].append(to_dict(v_elem))
                else:
                    result[inflection.camelize(k, False)].append(v_elem)
        else:
            result[inflection.camelize(k, False)] = v
    return result



from typing import List

from mlplatform_lib.mlplatform.mlplatform_http_client import (
    MlPlatformHttpClient,
    MlPlatformRequestType,
)
from mlplatform_lib.dataclass.model import ModelAutomlDto, ModelAutomlBestDto
from mlplatform_lib.utils.dataclass_utils import from_dict, to_dict

from mlplatform_lib.dataclass.experiment import ExperimentAutomlDto

from mlplatform_lib.dataclass.inference.inference_automl_dto import InferenceAutomlDto


class AutomlHttpClient(MlPlatformHttpClient):
    def __init__(self, mlplatform_addr, api_client):
        super().__init__(mlplatform_addr=mlplatform_addr, api_client=api_client)

    def get_models(self, experiment_id: int, train_id: int) -> List[ModelAutomlDto]:
        res = self.send_request(
            "models", {"experiments": experiment_id, "trains": train_id}, {}, {}, MlPlatformRequestType.READ,
        )
        model_infos = []
        for model_info in res.data:
            model_infos.append(from_dict(ModelAutomlDto, model_info))
        return model_infos

    def get_experiment(self, experiment_id: int) -> ExperimentAutomlDto:
        result = self.send_request("", {"experiments": experiment_id}, {}, {}, MlPlatformRequestType.READ)
        return from_dict(ExperimentAutomlDto, result.data)

    def get_inference(self, experiment_id: int, inference_id: int) -> InferenceAutomlDto:
        result = self.send_request(
            "",
            {"experiments": experiment_id, "inferences": inference_id},
            {},
            {},
            MlPlatformRequestType.READ,
        )
        return from_dict(InferenceAutomlDto, result.data)

    def update_inference(
        self, experiment_id: int, inference_automl_dto: InferenceAutomlDto
    ) -> InferenceAutomlDto:
        result = self.send_request(
            "inferences",
            {"experiments": experiment_id},
            {},
            to_dict(inference_automl_dto),
            MlPlatformRequestType.UPDATE,
        )
        return from_dict(InferenceAutomlDto, result.data)

    def insert_model(self, experiment_id: int, train_id: int, model: ModelAutomlDto) -> ModelAutomlDto:
        res = self.send_request(
            "models",
            {"experiments": experiment_id, "trains": train_id},
            {},
            to_dict(model),
            MlPlatformRequestType.CREATE,
        )
        return from_dict(ModelAutomlDto, res.data)

    def insert_model_automl_best(
        self, experiment_id: int, train_id: int, model_automl_best: ModelAutomlBestDto,
    ) -> ModelAutomlBestDto:
        result = self.send_request(
            "model-automl-bests",
            {"experiments": experiment_id, "trains": train_id},
            {},
            to_dict(model_automl_best),
            MlPlatformRequestType.CREATE,
        )
        return from_dict(ModelAutomlBestDto, result.data)
